==> leads-0.8.1.tar.gz: leads-0.8.1/leads_arduino/wheel_speed_sensor.py <==
from math import pi as _pi
from time import time as _time
from typing import override as _override

from leads import Device as _Device, get_device as _get_device
from leads_arduino.odometer import Odometer


def rpm2kmh(rpm: float, wheel_circumference: float) -> float:
    """
    :param rpm: revolutions per minute
    :param wheel_circumference: wheel circumference in centimeters
    :return: speed in kilometers per hour
    """
    return rpm * wheel_circumference * .0006


class WheelSpeedSensor(_Device):
    """
    See LEADS-Arduino.

    Supports:
    - Any hall effect sensor (switch)
    """

    def __init__(self, wheel_diameter: float, odometer_tag: str | None = None) -> None:
        super().__init__()
        self._wheel_circumference: float = wheel_diameter * 2.54 * _pi
        self._wheel_speed: float = 0
        self._last_valid: float = 0
        self._odometer_tag: str | None = odometer_tag
        self._odometer: Odometer | None = None

    @_override
    def initialize(self, *parent_tags: str) -> None:
        super().initialize(*parent_tags)
        if self._odometer_tag:
            self._odometer = _get_device(self._odometer_tag)

    @_override
    def update(self, data: str) -> None:
        if data.startswith(self._tag):
            self._wheel_speed = rpm2kmh(float(data[data.find(":") + 1:]), self._wheel_circumference)
            self._last_valid = _time()
            self._odometer.write(self._wheel_circumference * .00001)

    @_override
    def read(self) -> float:
        """
        :return: speed in kilometers per hour
        """
        # add .001 to avoid zero division
        d = rpm2kmh(60 / (.001 + _time() - self._last_valid), self._wheel_circumference)
        return 0 if d < 1 else d if d < 5 else self._wheel_speed

==> leads-0.8.9.tar.gz: leads-0.8.9/leads_arduino/wheel_speed_sensor.py <==
from time import time as _time
from typing import override as _override

from numpy import pi as _pi

from leads import Device as _Device, get_device as _get_device, Odometer as _Odometer


def rpm2kmh(rpm: float, wheel_circumference: float) -> float:
    """
    :param rpm: revolutions per minute
    :param wheel_circumference: wheel circumference in centimeters
    :return: speed in kilometers per hour
    """
    return rpm * wheel_circumference * .0006


class WheelSpeedSensor(_Device):
    """
    See LEADS-Arduino.

    Supports:
    - Any hall effect sensor (switch)
    """

    def __init__(self, wheel_diameter: float, odometer_tag: str | None = None) -> None:
        super().__init__()
        self._wheel_circumference: float = wheel_diameter * 2.54 * _pi
        self._wheel_speed: float = 0
        self._last_valid: float = 0
        self._odometer_tag: str | None = odometer_tag
        self._odometer: _Odometer | None = None

    @_override
    def initialize(self, *parent_tags: str) -> None:
        super().initialize(*parent_tags)
        if self._odometer_tag:
            self._odometer = _get_device(self._odometer_tag)

    @_override
    def update(self, data: str) -> None:
        if data.startswith(self._tag):
            self._wheel_speed = rpm2kmh(float(data[data.find(":") + 1:]), self._wheel_circumference)
            self._last_valid = _time()
            if self._odometer:
                self._odometer.write(self._wheel_circumference * .00001)

    @_override
    def read(self) -> float:
        """
        :return: speed in kilometers per hour
        """
        # add .001 to avoid zero division
        r = rpm2kmh(60 / (.001 + _time() - self._last_valid), self._wheel_circumference)
        return 0 if r < 1 else r if r < 5 else self._wheel_speed
